;;; framepop.el --- display temporary buffers in a dedicated frame

;; Copyright (C) 1993 Free Software Foundation, Inc.

;; Author: David Smith <dsmith@stats.adelaide.edu.au>
;; Maintainer: David Smith <dsmith@stats.adelaide.edu.au>
;; Created: 8 Oct 1993
;; Version: 1.0
;; Keywords: help

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:

;; Defines temp-buffer-show-function to display buffers in a dedicated
;; temporary frame (and so requires a display which can accomodate
;; separate frames). The frame is automatically shrink-wrapped to just
;; contain the buffer (restricted to a maximum and minimum
;; size). Buffers thus affected include *Help*, completion buffer and
;; buffer listings.

;; To make use of this package, place this file in your load-path and
;; add the following line to your .emacs:

;;   (cond (window-system (require 'framepop))

;; User variables are `framepop-max-frame-size', `framepop-min-frame-size'
;; and `framepop-frame-parameters'.

;; Six user functions are defined: you may want to bind them (the
;; first two are particularly useful) to keys with (for example) the
;; following commands in your .emacs:

;;   (define-key global-map "\C-cz" 'framepop-toggle-frame)
;;   (define-key global-map "\C-cv" 'framepop-scroll-frame)
;;   (define-key global-map "\C-cs" 'framepop-show-frame)
;;   (define-key global-map "\C-cx" 'framepop-iconify-frame)
;;   (define-key global-map "\C-cr" 'framepop-raise-frame)
;;   (define-key global-map "\C-cl" 'framepop-lower-frame)

;;; Bugs: 
;; 1. For some weird reason, if framepop-toggle-frame is bound to a
;;    single-event key, you need to press that key *twice* to
;;    deiconify the frame.
;; 2. M-C-v doesn't work on the temp frame as the message
;;    would suggest.
;; 3. It'd be nice to make the popup frame invisible instead of simply
;;    iconifying it, but make-frame-invisible is buggy...

;;; Code:

(defvar framepop-max-frame-size 35
  "*Maximum height of the FramePop frame")

(defvar framepop-min-frame-size 5
  "*Minimum height of the FramePop frame")

(defvar framepop-frame-parameters '((name . "FRAMEPOP") 
				    (unsplittable . t)
				    (autoraise . t)
				    (menu-bar-lines . 0)
				    (minibuffer . nil))
  "Default parameters used in with the FramePop frame")

(defvar framepop-last-displayed-buffer nil
  "Name of last buffer displayed in temp frame")

(defvar framepop-frame nil)

(defun framepop-display-buffer (buf)
  "Display-buffer for FramePop"
  (let ((oframe (selected-frame))
	(lines))
    (if (not (frame-live-p framepop-frame))
	(setq framepop-frame 
	      (make-frame framepop-frame-parameters)))
    ;; (make-frame-invisible framepop-frame) ; buggy
    (save-excursion
      (set-buffer buf)
      (setq lines 
	    (max
	     (min framepop-max-frame-size 
		  ;; +2 for mode line/minibuffer
		  (+ (count-lines (point-min) (point-max)) 2))
	     framepop-min-frame-size)))
    (modify-frame-parameters framepop-frame (list (cons 'height lines)))
    (select-frame framepop-frame)
    (make-frame-visible)
    (delete-other-windows)
    (switch-to-buffer buf t)
    (setq framepop-last-displayed-buffer (buffer-name buf))
    (raise-frame framepop-frame)
    (select-frame oframe)))

(defun framepop-iconify-frame nil
  "Iconify the FramePop frame"
  (interactive)
  (if (frame-live-p framepop-frame)
      (iconify-frame framepop-frame)
    (message "Popper frame deleted")))

(defun framepop-show-frame nil
  "Force the FramePop frame to be visible"
  (interactive)
  (let ((buf (or
	      (get-buffer framepop-last-displayed-buffer)
	      (get-buffer "*Help*"))))
    (if buf
	(framepop-display-buffer buf)
      (message "Last displayed temporary buffer has been killed."))))

(defun framepop-toggle-frame nil
  "Iconify or deiconify the FramePop frame"
  (interactive)
  (if (frame-live-p framepop-frame)
      (let ((oframe (selected-frame)))
	(select-frame framepop-frame)
	(iconify-or-deiconify-frame)
	(select-frame oframe))
    (message "No active popper frame")))
      
(defun framepop-scroll-frame (n)
  "Like scroll-other-window, but scrolls the window in the FramePop frame"
  (interactive "P")
  (framepop-show-frame)
  (save-window-excursion
    (select-window (frame-root-window framepop-frame))
    (scroll-up n)))

(defun framepop-lower-frame nil
  "Lower the FramePop frame"
  (interactive)
  (if (frame-live-p framepop-frame)
      (lower-frame framepop-frame)
    (message "No active FramePop frame")))

(defun framepop-raise-frame nil
  "Raise the FramePop frame"
  (interactive)
  (if (frame-live-p framepop-frame)
      (raise-frame framepop-frame)
    (message "No active FramePop frame")))

(if temp-buffer-show-function 
    (message "Warning: framepop.el has redefined temp-buffer-show-function"))

(setq temp-buffer-show-function 'framepop-display-buffer)

(provide 'framepop)

;;; framepop.el ends here


